var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  default: () => CreateDynamicImportComponentsPlugin
});
module.exports = __toCommonJS(src_exports);

// src/transform.ts
var import_parser = require("@babel/parser");
var import_ast_walker_scope = require("ast-walker-scope");
function isCallOf(node, test) {
  return !!(node && node.type === "CallExpression" && node.callee.type === "Identifier" && (typeof test === "string" ? node.callee.name === test : Array.isArray(test) ? test.includes(node.callee.name) : test(node.callee.name)));
}
function getTransformResult(s, id) {
  if (s?.hasChanged()) {
    return {
      code: s.toString(),
      get map() {
        return s.generateMap({
          source: id,
          includeContent: true,
          hires: true
        });
      }
    };
  }
}
function parseAst(code, lang = "") {
  const plugins = [];
  if (/tsx?$/.test(lang)) {
    plugins.push("typescript");
  }
  if (lang.endsWith("x")) {
    plugins.push("jsx");
  }
  return (0, import_parser.parse)(code, {
    sourceType: "module",
    plugins
  });
}
function transformAst(ast, s, option, offset = 0) {
  (0, import_ast_walker_scope.walkAST)(ast, {
    enter(node) {
      if (isCallOf(node, "dynamicImportComponents")) {
        const [type, ...args] = node.arguments.map((node2) => {
          if (node2.type === "StringLiteral") {
            return node2.value;
          } else {
            throw new SyntaxError("dynamicImportComponents argument must be string literal");
          }
        });
        if (Reflect.has(option, type)) {
          s.overwriteNode(node, option[type](...args), { offset });
        } else {
          throw new Error(`dynamicImportComponents type "${type}" not found`);
        }
      }
    }
  });
}

// src/common.ts
var REGEX_TS_FILE = /\.[cm]?[jt]sx?$/;
var REGEX_SETUP_SFC = /\.setup\.[cm]?[jt]sx?$/;
var REGEX_VUE_SFC = /\.vue$/;

// src/index.ts
var import_pluginutils = require("@rollup/pluginutils");
var import_magic_string_ast = require("magic-string-ast");
var import_compiler_sfc = require("@vue/compiler-sfc");
function CreateDynamicImportComponentsPlugin(option) {
  const filter = (0, import_pluginutils.createFilter)([REGEX_TS_FILE, REGEX_SETUP_SFC, REGEX_VUE_SFC]);
  return {
    name: "sort-dynamic-import",
    enforce: "pre",
    transform(code, id) {
      if (!filter(id))
        return;
      if (!code.includes("dynamicImportComponent"))
        return;
      const s = new import_magic_string_ast.MagicString(code);
      if (REGEX_VUE_SFC.test(id)) {
        const { descriptor } = (0, import_compiler_sfc.parse)(code, { filename: id });
        if (descriptor.scriptSetup) {
          const setUpAst = parseAst(descriptor.scriptSetup.content, descriptor.scriptSetup.lang);
          transformAst(setUpAst, s, option, descriptor.scriptSetup.loc.start.offset);
        }
        if (descriptor.script) {
          const scriptAst = parseAst(descriptor.script.content, descriptor.script.lang);
          transformAst(scriptAst, s, option, descriptor.script.loc.start.offset);
        }
      } else {
        let lang = /\.tsx?$/.test(id) ? "ts" : "js";
        if (id.endsWith("x")) {
          lang += "x";
        }
        const ast = parseAst(code, lang);
        transformAst(ast, s, option);
      }
      return getTransformResult(s, id);
    }
  };
}
